@@ -1,3 +1,21 @@
+# Copyright 2010-2013 Meik Michalke <meik.michalke@hhu.de>
+#
+# This file is part of the R package koRpus.
+#
+# koRpus is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# koRpus is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with koRpus.  If not, see <http://www.gnu.org/licenses/>.
+
+
 #' Guess language a text is written in
 #' 
 #' This function tries to guess the language a text is written in.
@@ -18,6 +36,7 @@
 #' @param quiet Logical. If \code{FALSE}, short status messages will be shown.
 #' @param in.mem Logical. If \code{TRUE}, the gzip compression will remain in memory (using \code{memCompress}), which
 #'		is probably the faster method. Otherwise temporary files are created and automatically removed on exit.
+#' @param format Either "file" or "obj". If the latter, \code{txt.file} is not interpreted as a file path but the text to analyze itself.
 #' @return An object of class \code{\link[koRpus]{kRp.lang-class}}.
 # @author m.eik michalke \email{meik.michalke@@hhu.de}
 #' @references
@@ -36,15 +55,20 @@
 #' }
 #' @export
 
-guess.lang <- function(txt.file, udhr.path, comp.length=300, keep.udhr=FALSE, quiet=TRUE, in.mem=TRUE){
+guess.lang <- function(txt.file, udhr.path, comp.length=300, keep.udhr=FALSE, quiet=TRUE, in.mem=TRUE, format="file"){
 
 	# try to read the file
 	if(file.exists(txt.file)){
 		txt <- paste(scan(txt.file, what=character(), quiet=quiet), collapse=" ")
+	} else if(identical(format, "obj")) {
+		txt <- txt.file
 	} else {
 		stop(simpleError(paste("File not found:", txt.file)))
 	}
 
+	# force text into UTF-8 format
+	txt <- enc2utf8(txt)
+
 	# read the declarations into a data.frame
 	udhr <- read.udhr(udhr.path, quiet=quiet)
 
@@ -60,19 +84,19 @@ guess.lang <- function(txt.file, udhr.path, comp.length=300, keep.udhr=FALSE, qu
 		txt.short <- paste(text.sliced[1:comp.length], collapse="")
 	}
 
-	udhr.comressed <- sapply(1:dim(udhr)[1], function(num.udhr){
-			curr.udhr <- udhr[num.udhr,]
-			if(!isTRUE(quiet)){
-				cat(paste0("Comparing text to ", curr.udhr["name"]), "...\n", sep="")
-			} else {}
-			udhr.plain <- paste(curr.udhr["text"])
-			udhr.plus.text <- paste(curr.udhr["text"], txt.short, collapse="")
-			udhr.plain.gz <- txt.compress(udhr.plain, in.mem=in.mem)
-			udhr.plus.text.gz <- txt.compress(udhr.plus.text, in.mem=in.mem)
-			compression.diff <- udhr.plus.text.gz$gz.size - udhr.plain.gz$gz.size
-			return(compression.diff)
-		}
-	)
+	udhr.comressed <- c()
+	for (num.udhr in 1:dim(udhr)[1]){
+		curr.udhr <- udhr[num.udhr,]
+		if(!isTRUE(quiet)){
+			cat(paste0("Comparing text to ", curr.udhr["name"]), "...\n", sep="")
+		} else {}
+		udhr.plain <- paste(curr.udhr["text"])
+		udhr.plus.text <- paste(curr.udhr["text"], txt.short, collapse="")
+		udhr.plain.gz <- txt.compress(udhr.plain, in.mem=in.mem)
+		udhr.plus.text.gz <- txt.compress(udhr.plus.text, in.mem=in.mem)
+		compression.diff <- udhr.plus.text.gz$gz.size - udhr.plain.gz$gz.size
+		udhr.comressed <- c(udhr.comressed, compression.diff)
+	}
 
 	# check if to throw out all the full declarations
 	if(isTRUE(keep.udhr)){
@@ -81,9 +105,11 @@ guess.lang <- function(txt.file, udhr.path, comp.length=300, keep.udhr=FALSE, qu
 		udhr.results <- subset(udhr, select=-text)
 	}
 	# add the diff
-	udhr.results$diff <- udhr.comressed
+	udhr.results[["diff"]] <- udhr.comressed
+	udhr.results[["diff.std"]] <- scale(udhr.comressed)
+
 	# sort results by diff
-	udhr.results <- udhr.results[order(udhr.results$diff),]
+	udhr.results <- udhr.results[order(udhr.results[["diff"]]),]
 	dimnames(udhr.results)[[1]] <- 1:length(dimnames(udhr.results)[[1]])
 
 	# get the best match
